/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkLookupTableToTexture.cxx,v $
  Language:  C++
  Date:      $Date: 2000/02/04 17:07:13 $
  Version:   $Revision: 1.35 $
  Thanks:    Thanks to David Pont at Forest Research, New Zealand who
             developed and contributed this class.


Copyright (c) 1993-2003 Ken Martin, Will Schroeder, Bill Lorensen 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

 * Neither name of Ken Martin, Will Schroeder, or Bill Lorensen nor the names
   of any contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

 * Modified source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/
#include "vtkLookupTableToTexture.h"
#include "vtkMath.h"
//#include "vtkTCoords.h"
#include "vtkObjectFactory.h"
#include "vtkLookupTable.h"
#include "vtkStructuredPoints.h"
#include "vtkPointData.h"

#include <vtkTIFFWriter.h>	// !!!
//#include <vtkBMPWriter.h>	// !!!
#include <vtkImageData.h>	// !!!

//------------------------------------------------------------------------------
vtkLookupTableToTexture* vtkLookupTableToTexture::New()
{
  // First try to create the object from the vtkObjectFactory
  vtkObject* ret = vtkObjectFactory::CreateInstance("vtkLookupTableToTexture");
  if(ret)
    {
    return (vtkLookupTableToTexture*)ret;
    }
  // If the factory was unable to create the object, then create it here.
  return new vtkLookupTableToTexture;
}


// Construct with s,t range=(0,1)
vtkLookupTableToTexture::vtkLookupTableToTexture()
{
  this->ScalarRange[0] = 0.0;
  this->ScalarRange[1] = 1.0;

  // s,t
  this->TextureDimensions[0] = 256;
  this->TextureDimensions[1] = 1;

  this->LookupTable = NULL;

  this->SRange[0] = 0.0;
  this->SRange[1] = 1.0;

  this->TRange[0] = 0.0;
  this->TRange[1] = 1.0;
}

vtkLookupTableToTexture::~vtkLookupTableToTexture()
{
  if( this->LookupTable )
    {
    this->LookupTable->UnRegister(this);
    }
}

// Overload standard modified time function. If lookup table is modified,
// then this object is modified as well.
unsigned long vtkLookupTableToTexture::GetMTime()
{
  unsigned long mTime=this->MTime.GetMTime();
  unsigned long lutMTime;

  if( this->LookupTable != NULL )
    {
    lutMTime = this->LookupTable->GetMTime();
    mTime = ( lutMTime > mTime ? lutMTime : mTime );
    }

  return mTime;
}

// Specify a lookup table for creating the texture.
void vtkLookupTableToTexture::SetLookupTable(vtkLookupTable *lut)
{
  if ( this->LookupTable != lut )
    {
    if ( this->LookupTable)
      {
      this->LookupTable->UnRegister(this);
      }
    this->LookupTable = lut;
    if (lut)
      {
      lut->Register(this);
      }
    this->Modified();
    }
}

vtkLookupTable *vtkLookupTableToTexture::GetLookupTable()
{
  if ( this->LookupTable == NULL )
    {
    this->CreateDefaultLookupTable();
    }
  return this->LookupTable;
}

void vtkLookupTableToTexture::CreateDefaultLookupTable()
{
  if ( this->LookupTable)
    {
    this->LookupTable->UnRegister(this);
    }
  this->LookupTable = vtkLookupTable::New();
  // 8 colors, black and white
  this->LookupTable->SetHueRange( 0, 0 );
  this->LookupTable->SetSaturationRange( 0, 0 );
  this->LookupTable->SetValueRange( 0, 1 );
  this->LookupTable->SetAlphaRange( 1, 1 );
  this->LookupTable->SetRange( this->ScalarRange );
//  this->LookupTable->SetNumberOfColors( 8 );	use lut default
  this->LookupTable->Build();
}


void vtkLookupTableToTexture::Execute()
{
  if ( this->LookupTable == NULL )
    {
    this->CreateDefaultLookupTable();
    }

  if( this->LookupTable->GetMTime() > this->MTime.GetMTime() )
    {
	  this->LookupTable->Build();
    }

  int dim[3], t, s;
  unsigned char *outP;
	unsigned char *rgba;
	double v, *range = this->LookupTable->GetRange();

  vtkDebugMacro(<<"Generating texture!");

  dim[0] = this->TextureDimensions[0];
  dim[1] = this->TextureDimensions[1];
  dim[2] = 1;
  vtkStructuredPoints *output = this->GetOutput();
  output->SetDimensions( dim );
  output->SetNumberOfScalarComponents( 4 );

//  vtkScalars *tScalars = vtkScalars::New();
  vtkUnsignedCharArray *tScalars = vtkUnsignedCharArray::New();
  //tScalars->SetDataType( VTK_UNSIGNED_CHAR );
  tScalars->SetNumberOfComponents( 4 );
  //tScalars->SetNumberOfScalars( dim[0]*dim[1] );
  tScalars->SetNumberOfTuples( dim[0]*dim[1] );

  outP = (unsigned char *)tScalars->GetVoidPointer( 0 );

  for( s=0; s<dim[0]; s++ )
  	{
    v = range[0] + (range[1] - range[0])* ((double)s /(dim[0]-1.0));
		rgba = this->LookupTable->MapValue( v );

	  for( t=0; t<dim[1]; t++ )
    	{
      // set texture pixel rgba
      *outP++ = rgba[0];
      *outP++ = rgba[1];
      *outP++ = rgba[2];
      *outP++ = rgba[3];
      }
    }

  output->GetPointData()->SetScalars( tScalars );
  tScalars->Delete();


// does not work because execute must be called!    this->Update();
// !!! write texture to file
vtkImageData *id1 = vtkImageData::New();
  id1->SetScalarType( VTK_UNSIGNED_CHAR );
  id1->SetDimensions( dim );
  id1->SetNumberOfScalarComponents( 4 );
	id1->GetPointData()->SetScalars( tScalars );
vtkTIFFWriter *tw1 = vtkTIFFWriter::New();
//vtkBMPWriter *tw1 = vtkBMPWriter::New();
	tw1->SetInput( id1 );
  tw1->SetFileName( "t.tif" );
//  tw1->SetFileName( "t.bmp" );
  tw1->SetFileDimensionality( 2 );
  tw1->Write();

}

void vtkLookupTableToTexture::PrintSelf(ostream& os, vtkIndent indent)
{
  vtkStructuredPointsSource::PrintSelf(os,indent);
  //!!! update

  if ( this->LookupTable )
    {
    os << indent << "Lookup Table:\n";
    this->LookupTable->PrintSelf(os,indent.GetNextIndent());
    }
  else
    {
    os << indent << "Lookup Table: (none)\n";
    }

  os << indent << "Scalar Range: (" << this->ScalarRange[0] << ", "
                                    << this->ScalarRange[1] << ")\n";

  os << indent << "S Range: (" << this->SRange[0] << ", "
                               << this->SRange[1] << ")\n";
  os << indent << "T Range: (" << this->TRange[0] << ", "
                               << this->TRange[1] << ")\n";

}

