/*=========================================================================

  Program:   Visualization Toolkit
  Module:    $RCSfile: vtkLookupTableToTexture.h,v $
  Language:  C++
  Date:      $Date: 2000/02/04 17:07:13 $
  Version:   $Revision: 1.28 $
  Thanks:    Thanks to David Pont at Forest Research, New Zealand who
             developed and contributed this class.

Copyright (c) 1993-2003 Ken Martin, Will Schroeder, Bill Lorensen 
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice,
   this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

 * Neither name of Ken Martin, Will Schroeder, or Bill Lorensen nor the names
   of any contributors may be used to endorse or promote products derived
   from this software without specific prior written permission.

 * Modified source versions must be plainly marked as such, and must not be
   misrepresented as being the original software.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/
// .NAME vtkLookupTableToTexture - generates texture coordinates mapping scalar values
// into a texture. The default texture renders scalar data in discrete colour bands.
// .SECTION Description
// vtkLookupTableToTexture 
// vtkLookupTableToTexture provides a texture based alternative to using vtkLookupTable
// and vtkMapper to map scalar values to colour.
// At a minimum you should call SetInput and SetScalarRange.
// Pass the output to a vtkDataSetMapper using SetInput and call ScalarVisibilityOff().
// Pass the mapper output to vtkActor using SetMapper, and pass the stipple texture
// with SetTexture( vtkLookupTableToTexture->GetTexture() ).
// By default a black and white stipple texture is created using an internal
// vtkLookupTable with: SetHueRange( 0, 0 ) SetSaturationRange( 0, 0 )
// SetValueRange( 0, 1 ) SetNumberOfColors( 2 ).
// You provide your own vtkLookupTable to control color.
// The stipple pattern is created using the first and last colors in the lookup table.
// Contour bands can be enabled by setting ContourBandWidth > 0. These are solid
// bands of colour in the texture. The number of bands is set to the vtkLookupTable
// NumberOfColors. Those bands provide a basic visualisation of scalar contours.
// ReverseContourBands can be used to create light contour bands on dark stipple
// and vice versa to help distinguish the bands from the stipple pattern.
// ContourBandWidth can be used to create a texture consisting of bands of solid
// color, with no stipple pattern.


// !!! check how to make alpha work

// .SECTION See Also
// vtkDataSetMapper vtkActor vtkTexture vtkLookupTable

#ifndef __vtkLookupTableToTexture_h
#define __vtkLookupTableToTexture_h

#include <vtkStructuredPointsSource.h>
class vtkLookupTable;

class VTK_EXPORT vtkLookupTableToTexture : public vtkStructuredPointsSource
{
public:
  vtkTypeMacro(vtkLookupTableToTexture,vtkStructuredPointsSource);
  void PrintSelf(ostream& os, vtkIndent indent);

  // Description:
  // Construct with... !!! update
  static vtkLookupTableToTexture *New();

  // Description:
  // Overload standard modified time function. If lookup table is modified,
  // then this object is modified as well.
  unsigned long GetMTime();

  // Description:
  // Set the texture dimensions (s,t) in pixels.
  // A power of 2 is required for OpenGL. vtk will resample to a power of 2 if
  // necessary, but this causes blending across texture band boundaries (in the s direction).
  // Low s values can cause irregular band widths.
  // The default is 64,1.
  vtkSetVector2Macro(TextureDimensions,double);
  vtkGetVectorMacro(TextureDimensions,double,2);

  // Description:
  // Specify a lookup table for creating the texture.
	// The default black and white lookup table is created with:
  // SetHueRange( 0, 0 ) SetSaturationRange( 0, 0 )
	// SetValueRange( 0, 1 ) SetNumberOfColors( 8 ).
  // Use SetHueRange to get colors.
  // Use SetNumberOfColors to change the number of texture bands.
  void SetLookupTable(vtkLookupTable *lut);
  vtkLookupTable *GetLookupTable();

  // !!! drop s and t ranges? or useful with hatch patterns?
  // Description:
  // Specify s-coordinate range for texture s-t coordinate pair.
  vtkSetVector2Macro(SRange,double);
  vtkGetVectorMacro(SRange,double,2);

  // Description:
  // Specify t-coordinate range for texture s-t coordinate pair.
  vtkSetVector2Macro(TRange,double);
  vtkGetVectorMacro(TRange,double,2);

protected:
  vtkLookupTableToTexture();
  ~vtkLookupTableToTexture();
  vtkLookupTableToTexture(const vtkLookupTableToTexture&) {};
  void operator=(const vtkLookupTableToTexture&) {};

  void Execute();

  // Description:
  // Create default lookup table.
  // Used to create one when none is provided.	!!! make protected?
  void CreateDefaultLookupTable();

  double ScalarRange[2];

  double SRange[2];
  double TRange[2];

  double TextureDimensions[2];

  vtkLookupTable *LookupTable;
};

#endif


